DOM_BOX_TUPLE = (2.6, 2.6)
DOM_BOX_TUPLE_LIST_FOR_TWO = ((2.6, 1.3), (2.6, 1.3))
DOM_BOX_TUPLE_LIST_FOR_THREE = ((2.7, 0.9), (2.7, 0.9), (2.7, 0.9))

# color from -> to (like douban color)
COLOR_TUPLE = (("#CAE3F7", "#266EDA"), ("#C9E8AF", "#8FD670"), ("#FEE0D7", "#F27A40"))

DOM_BOX_DICT = {
    1: {"dom": (DOM_BOX_TUPLE,)},
    2: {"dom": DOM_BOX_TUPLE_LIST_FOR_TWO},
    3: {"dom": DOM_BOX_TUPLE_LIST_FOR_THREE},
}

DEFAULT_DOM_COLOR = "#444444"
MONTH_NAMES = [
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec",
]

TYPE_INFO_DICT = {
    "duolingo": "Duolingo",
    "github": "GitHub",
    "leetcode": "LeetCode",
    "twitter": "Twitter",
    "youtube": "YouTube",
    "gitlab": "GitLab",
    "strava": "Strava",
    "bilibili": "bilibili",
    "nike": "Nike",
    "strava": "Strava",
    "ns": "NS",
    "kindle": "Kindle",
    "garmin": "Garmin",
    "shanbay": "Shanbay",
    "forest": "Forest",
    "jike": "Jike",
}
